@@ -63,8 +63,14 @@ my $stat;
 if (defined $load) {
 	eval { require JSON::XS; 1 }
 		or die "JSON::XS is required for --load option to work\n";
+	my $fd;
+	if ($load eq '-') {
+		$fd = \*STDIN;
+	} else {
+		open (my $fd, "<", $load)
+			or die "Failed to r-open $load: $!";
+	};
 	local $/;
-	open (my $fd, "<", $load) or die "Failed to r-open $load: $!";
 	defined (my $json = <$fd>) or die "Failed to read from $load: $!";
 	close $fd;
 	my $raw = JSON::XS::decode_json($json);
@@ -72,8 +78,9 @@ if (defined $load) {
 } else {
 	$stat = Statistics::Descriptive::LogScale->new(
 		base => $opt{base}, linear_width => $opt{zero});
+	my $re_num = qr/(?:[-+]?(?:\d+\.?\d*|\.\d+)(?:[Ee][-+]?\d+)?)/;
 	while (<STDIN>) {
-		$stat->add_data(/(-?\d+(?:\.\d*)?)/g);
+		$stat->add_data(/($re_num)/g);
 	};
 };
 
@@ -84,12 +91,7 @@ if (!$stat->count) {
 
 # Let's do the real work
 my ($width, $height) = @opt{"width", "height"};
-my $hist = $stat->histogram( %opt, count => $width);
-
-# Find maximum or heights
-my $max_val = $hist->[0][0];
-$_->[0] > $max_val and $max_val = $_->[0] for @$hist;
-$_->[0] /= $max_val for @$hist;
+my $hist = $stat->histogram( %opt, count => $width, normalize_to => 1);
 
 # draw!
 my $gd = GD::Simple->new($width, $height);
