@@ -6,16 +6,16 @@ package Statistics::Descriptive::LogScale;
 
 =head1 NAME
 
-Statistics::Descriptive::LogScale - Memory-efficient approximate descriptive
-statistics class.
+Statistics::Descriptive::LogScale - Memory-efficient approximate univariate
+descriptive statistics class.
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =cut
 
-our $VERSION = 0.08;
+our $VERSION = 0.09;
 
 =head1 SYNOPSIS
 
@@ -61,7 +61,7 @@ with enough information to restore the original object.
 
     # Import into existing LogScale instance
     my $plain_hash = decoder_of_choice( $more_raw_data );
-    $copy_of_stat->add_data_hash( $more_raw_data->{data} );
+    $copy_of_stat->add_data_hash( $plain_hash->{data} );
 
 =head2 Histograms
 
@@ -136,7 +136,7 @@ use fields qw(
 );
 
 # This is for infinite portability^W^W portable infinity
-my $inf = 9**9**9;
+my $INF = 9**9**9;
 
 =head2 new( %options )
 
@@ -170,14 +170,16 @@ so that borders of linear and logarithmic bins fit nicely.
 Used for cloning.
 See C<add_data_hash()>.
 
-=item * linear_thresh - absolute value threshold below which everything is
+=item * zero_thresh - absolute value threshold below which everything is
 considered zero.
-DEPRECATED, C<linear_width> and C<linear_threshold> override this if given.
+DEPRECATED, C<linear_width> and C<linear_thresh> override this if given.
 
 =back
 
 =cut
 
+my @new_keys = qw( base linear_thresh linear_width zero_thresh data );
+	# TODO Throw if extra options given?
 sub new {
 	my $class = shift;
 	my %opt = @_;
@@ -243,7 +245,8 @@ follow the interface of L<Statistics::Descriptive> and co,
 with minor additions.
 
 All methods return C<undef> on empty data set, except for
-C<count>, C<sum>, C<sumsq> and C<variance> which all return 0.
+C<count>, C<sum>, C<sumsq>, C<stdandard_deviation> and C<variance>
+which all return 0.
 
 B<NOTE> This module caches whatever it calculates very agressively.
 Don't hesitate to use statistical functions (except for sum_of/mean_of)
@@ -275,6 +278,10 @@ The state of object is guaranteed to remain consistent in such case.
 
 B<NOTE> Cache is reset, even if no data was actually inserted.
 
+B<NOTE> It is possible to add infinite values to data pool.
+The module will try and calculate whatever can still be calculated.
+However, no portable way of serializing such values is done yet.
+
 =cut
 
 sub add_data {
@@ -366,36 +373,51 @@ sub mean {
 
 =head2 variance
 
+=head2 variance( $correction )
+
 Return data variance, i.e. E((x - E(x)) ** 2).
 
+Bessel's correction (division by n-1 instead of n) is used by default.
+This may be changed by specifying $correction explicitly.
+
+B<NOTE> The binning strategy used here should also introduce variance bias.
+This is not yet accounted for.
+
 =cut
 
 sub variance {
 	my $self = shift;
+	my $correction = shift;
 
-	# This part is stolen from Statistics::Descriptive
-	my $div = @_ ? 0 : 1;
-	if ($self->{count} < 1 + $div) {
-		return 0;
-	}
+	# in fact we'll receive correction='' because of how cache works
+	$correction = 1 unless defined $correction and length $correction;
+
+	return 0 if ($self->{count} < 1 + $correction);
 
 	my $var = $self->sumsq - $self->sum**2 / $self->{count};
-	return $var <= 0 ? 0 : $var / ( $self->{count} - $div );
+	return $var <= 0 ? 0 : $var / ( $self->{count} - $correction );
 };
 
 =head2 standard_deviation
 
+=head2 standard_deviation( $correction )
+
 =head2 std_dev
 
-Return standard deviation (square root of variance).
+Return standard deviation, i.e. square root of variance.
+
+Bessel's correction (division by n-1 instead of n) is used by default.
+This may be changed by specifying $correction explicitly.
+
+B<NOTE> The binning strategy used here should also introduce variance bias.
+This is not yet accounted for.
 
 =cut
 
 sub standard_deviation {
-	# This part is stolen from Statistics::Descriptive
 	my $self = shift;
-	return if (!$self->count());
-	return sqrt($self->variance());
+
+	return sqrt($self->variance(@_));
 };
 
 =head2 cdf ($x)
@@ -696,7 +718,7 @@ sub frequency_distribution_ref {
 		$index = [ map { $min + $_ * $step } 1..$index ];
 	};
 
-	@$index = (-$inf, sort { $a <=> $b } @$index);
+	@$index = (-$INF, sort { $a <=> $b } @$index);
 
 	my @count;
 	for (my $i = 0; $i<@$index-1; $i++) {
@@ -761,19 +783,25 @@ sub linear_threshold {
 
 =head2 add_data_hash ( { value => weight, ... } )
 
-Add values with weights. This can be used to import data from other
+Add values with counts/weights.
+This can be used to import data from other
 Statistics::Descriptive::LogScale object.
 
-Negative counts are treated as "forgetting" data.
-If a bin count goes below zero, such bin is simply discarded.
-Count is guaranteed to remain consistent in such case.
-
 Returns self, so that methods can be chained.
 
-If incorrect data is given (i.e. non-numeric, undef),
-an exception is thrown and nothing gets inserted.
+Negative counts are allowed and treated as "forgetting" data.
+If a bin count goes below zero, such bin is simply discarded.
+Minus infinity weight is allowed and has the same effect.
+Data is guaranteed to remain consistent.
 
-B<NOTE> Cache is reset, even if no data was actually inserted.
+If incorrect data is given (i.e. non-numeric, undef, or +infinity),
+an exception is thrown and nothing changes.
+
+B<NOTE> Cache may be reset, even if no data was actually inserted.
+
+B<NOTE> It is possible to add infinite values to data pool.
+The module will try and calculate whatever can still be calculated.
+However, no portable way of serializing such values is done yet.
 
 =cut
 
@@ -781,48 +809,81 @@ sub add_data_hash {
 	my $self = shift;
 	my $hash = shift;
 
-	delete $self->{cache};
-
-	# check incoming data for consistency
+	# check incoming data for being numeric, and no +inf values
 	eval {
 		use warnings FATAL => qw(numeric);
-		no warnings qw(void);
-		defined $_ and 0+$_ for %$hash;
+		while (my ($k, $v) = each %$hash) {
+			$k == 0+$k and $v == 0+$v and $v < $INF
+				or die "Infinite count for $k\n";
+		}
 	};
 	croak __PACKAGE__.": add_data_hash failed: $@"
 		if $@;
 
+	delete $self->{cache};
+
 	# update our counters
 	foreach (keys %$hash) {
 		next unless $hash->{$_};
 		my $key = $self->_round($_);
-		$self->{data}{$key} += $hash->{$_};
-		if ($self->{data}{$key} > 0) {
+
+		# Insert data. Make sure -Inf doesn't corrupt our counter.
+		my $newcount = ($self->{data}{$key} || 0) + $hash->{$_};
+		if ($newcount > 0) {
 			# normal insert
+			$self->{data}{$key} = $newcount;
 			$self->{count} += $hash->{$_};
 		} else {
-			# erase empty bin, adjust count
-			$self->{count} += $hash->{$_} - delete $self->{data}{$key};
+			# We're "forgetting" data, AND the bin got empty
+			$self->{count} -= delete $self->{data}{$key} || 0;
 		};
 	};
 	$self;
 };
 
-=head2 get_data_hash
+=head2 get_data_hash( %options )
 
 Return distribution hashref {value => number of occurances}.
 
 This is inverse of add_data_hash.
 
+Options may include:
+
+=over
+
+=item * min - ignore values below this. (See find_boundaries)
+
+=item * max - ignore values above this. (See find_boundaries)
+
+=item * ltrim - ignore this % of values on lower end. (See find_boundaries)
+
+=item * rtrim - ignore this % of values on upper end. (See find_boundaries)
+
+=item * noise_thresh - strip bins with count below this.
+
+=back
+
 =cut
 
 sub get_data_hash {
-	my $self = shift;
+	my ($self, %opt) = @_;
 
-	# shallow copy of data
-	my $hash = {%{ $self->{data} }};
-	return $hash;
+	# shallow copy of data if no options given
+	return {%{ $self->{data} }} unless %opt;
 
+	my ($min, $max) = $self->find_boundaries( %opt );
+	my $noize = $opt{noize_thresh} || 0;
+
+	my $data = $self->{data};
+	my %hash;
+	foreach (keys %$data ) {
+		$_ < $min and next;
+		$_ > $max and next;
+		$data->{$_} < $noize and next;
+		$hash{$_} = $data->{$_};
+	};
+
+	return \%hash;
 };
 
 =head2 TO_JSON()
@@ -838,16 +899,35 @@ B<NOTE> This module DOES NOT require JSON::XS or serialize to JSON.
 It just deals with data.
 Use C<JSON::XS>, C<YAML::XS>, C<Data::Dumper> or any serializer of choice.
 
-=head2 clone()
+    my $raw_data = $stat->TO_JSON;
+    Statistics::Descriptive::LogScale->new( %$raw_data );
+
+Would generate an exact copy of C<$stat> object
+(provided it's S::D::L and not a subclass).
+
+=head2 clone( [ %options ] )
 
 Copy constructor - returns copy of an existing object.
 Cache is not preserved.
 
+Constructor options may be given to override existing data. See new().
+
+Trim options may be given to get partial data. See get_data_hash().
+
 =cut
 
 sub clone {
-	my $self = shift;
-	return (ref $self)->new( %{ $self->TO_JSON } );
+	my ($self, %opt) = @_;
+
+	my $raw = $self->TO_JSON;
+	if (%opt) {
+		$raw->{data} = $self->get_data_hash( %opt )
+			unless exists $opt{data};
+		exists $opt{$_} and $raw->{$_} = $opt{$_}
+			for @new_keys;
+	};
+
+	return (ref $self)->new( %$raw );
 };
 
 sub TO_JSON {
@@ -938,8 +1018,8 @@ sub sum_of {
 		return $sum;
 	};
 
-	$realmin = -$inf unless defined $realmin;
-	$realmax =  $inf unless defined $realmax;
+	$realmin = -$INF unless defined $realmin;
+	$realmax =  $INF unless defined $realmax;
 	return 0 if( $realmin >= $realmax );
 
 	# correct limits. $min, $max are indices; $left, $right are limits
@@ -1006,6 +1086,9 @@ Options may include:
 
 =item * rtrim - ignore this % of values on upper end.
 
+=item * normalize_to <nnn> - adjust counts so that max number becomes nnn.
+This may be useful if one intends to draw pictures.
+
 =back
 
 Either count or index must be present.
@@ -1016,8 +1099,7 @@ for omitting sample tails and outputting pretty pictures.
 =cut
 
 sub histogram {
-	my $self = shift;
-	my %opt = @_;
+	my ($self, %opt) = @_;
 
 	return unless $self->count;
 	my ($min, $max) = $self->find_boundaries( %opt );
@@ -1044,6 +1126,16 @@ sub histogram {
 		my $count = $self->_count( $index[$i], $index[$i+1] );
 		push @ret, [ $count, $index[$i], $index[$i+1] ];
 	};
+
+	# if normalize - find maximum & divide by it
+	if (my $norm = $opt{normalize_to}) {
+		my $max = 0;
+		$max < $_->[0] and $max = $_->[0]
+			for @ret;
+		$norm /= $max;
+		$_->[0] *= $norm for @ret;
+	};
+
 	return \@ret;
 };
 
@@ -1055,9 +1147,9 @@ Options may include:
 
 =over
 
-=item * min - ignore values below this. default = max + epsilon.
+=item * min - ignore values below this. default = min() - epsilon.
 
-=item * max - ignore values above this. default = min - epsilon.
+=item * max - ignore values above this. default = max() + epsilon.
 
 =item * ltrim - ignore this % of values on lower end.
 
@@ -1140,12 +1232,12 @@ sub _memoize_method {
 }; # end of _memoize_method
 
 # Memoize all the methods w/o arguments
-foreach ( qw(sum sumsq mean min max standard_deviation mode) ) {
+foreach ( qw(sum sumsq mean min max mode) ) {
 	__PACKAGE__->_memoize_method($_);
 };
 
 # Memoize methods with 1 argument
-foreach ( qw(quantile central_moment std_moment) ) {
+foreach (qw(quantile central_moment std_moment standard_deviation variance)) {
 	__PACKAGE__->_memoize_method($_, 1);
 };
 
@@ -1281,18 +1373,26 @@ Konstantin S. Uvarin, C<< <khedin at gmail.com> >>
 
 =head1 BUGS
 
-The module is currently in alpha stage. There may be bugs.
+The module is currently under development. There may be bugs.
 
-mode() is unstable around zero, better algorithm wanted.
+C<mode()> only works for discrete distributions, and simply returns
+the first bin with largest bin count.
+A better algorithm is wanted.
 
-sum_of() requires more extensive unit testing.
+C<sum_of()> should have been made a private method.
+Its signature and/or name may change in the future.
 
-Adding linear interpolation could result in precision gains at a little
-performance cost.
+See the TODO file in the distribution package.
 
-Please report any bugs or feature requests to C<bug-statistics-descriptive-logscale at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Statistics-Descriptive-LogScale>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+Please feel free to post bugs and/or feature requests to github:
+L<https://github.com/dallaylaen/perl-Statistics-Descriptive-LogScale/issues/new>
+
+Alternatively, you can use CPAN RT
+via e-mail C<bug-statistics-descriptive-logscale at rt.cpan.org>,
+or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Statistics-Descriptive-LogScale>.
+
+Your contribution is appreciated.
 
 =head1 SUPPORT
 
@@ -1300,7 +1400,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Statistics::Descriptive::LogScale
 
-
 You can also look for information at:
 
 =over 4
@@ -1327,7 +1426,6 @@ L<http://search.cpan.org/dist/Statistics-Descriptive-LogScale/>
 
 =back
 
-
 =head1 ACKNOWLEDGEMENTS
 
 This module was inspired by a talk that Andrew Aksyonoff, author of
@@ -1342,7 +1440,7 @@ CountZero from http://perlmonks.org
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2013 Konstantin S. Uvarin.
+Copyright 2013-2015 Konstantin S. Uvarin.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
@@ -1350,7 +1448,6 @@ by the Free Software Foundation; or the Artistic License.
 
 See http://dev.perl.org/licenses/ for more information.
 
-
 =cut
 
 1; # End of Statistics::Descriptive::LogScale
